/*
 * BarcodeGenerator.java
 *
 * $Id: BarcodeGenerator.java,v 1.1 2008-06-26 10:42:17 mario Exp $
 */
package com.imagos.axilice.core.util;

import java.awt.image.BufferedImage;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;

import net.sourceforge.barbecue.Barcode;
import net.sourceforge.barbecue.BarcodeFactory;
import net.sourceforge.barbecue.BarcodeImageHandler;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * Generates the images representing barcodes with the help of the Barbecue
 * library. <br/> Responsibilities:
 * <ul>
 * <li>Generate barcode images. </li>
 * </ul>
 * <br/>
 * @author Copyright (c) 2007 Imagos 2005 S.L.
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * <br/>
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <br/>
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
public class BarcodeGenerator {
	/**
	 * Create a new Barcode image coded from the barcode data.
	 * @param barcode String, barcode data with the number the barcode
	 * will be generated for.
	 * @param display Display, SWT Display instance used to build the GUI.
	 * @return Image, image object with the barcode generated.
	 */
	public static Image createBarcodeImage(String barcode, Display display) {
		Image img = null;
		try {
			Barcode bc = BarcodeFactory.createEAN128(barcode);
			ImageData imgData = convertToSWT(BarcodeImageHandler.getImage(bc));
			
			img = new Image(display, imgData);
		} // try
		catch (Exception ex) {
			ex.printStackTrace();
			img = null;
		}

		return img;
	}
	
	/**
	 * Create a new Barcode image coded from the barcode data.
	 * @param barcode String, barcode data with the number the barcode
	 * will be generated for.
	 * @param barcodeType String, barcode type to generate.
	 * @param display Display, SWT Display instance used to build the GUI.
	 * @return Image, image object with the barcode generated.
	 */
	public static Image createBarcodeImage(String barcode, String barcodeType, Display display) {
		Image img = null;
		try {
			Barcode bc = BarcodeFactory.createEAN128(barcode);
			ImageData imgData = convertToSWT(BarcodeImageHandler.getImage(bc));
			
			img = new Image(display, imgData);
		} // try
		catch (Exception ex) {
			ex.printStackTrace();
			img = null;
		}

		return img;
	}

	/**
	 * Converts an AWT BufferedImage to SWT format. The library Barbecue, used
	 * for the barcode generation, only generates images in AWT format. So
	 * a mechanism for converting them into the SWT (GUI library used
	 * in the application) is needed.
	 * @param bufferedImage BufferedImage, AWT BufferedImage to convert.
	 * @return ImageData, SWT formatted image.
	 * <br/>
	 * Copyright (c) 2000, 2004 IBM Corporation and others.
	 * All rights reserved. This program and the accompanying materials
	 * are made available under the terms of the Eclipse Public License v1.0
	 * which accompanies this distribution, and is available at
	 * http://www.eclipse.org/legal/epl-v10.html
	 *
	 * Contributors:
	 *     IBM Corporation - initial API and implementation
	 */
	private static ImageData convertToSWT(BufferedImage bufferedImage) {
		if (bufferedImage.getColorModel() instanceof DirectColorModel) {
			DirectColorModel colorModel = (DirectColorModel) bufferedImage
					.getColorModel();
			PaletteData palette = new PaletteData(colorModel.getRedMask(),
					colorModel.getGreenMask(), colorModel.getBlueMask());
			ImageData data = new ImageData(bufferedImage.getWidth(),
					bufferedImage.getHeight(), colorModel.getPixelSize(),
					palette);
			WritableRaster raster = bufferedImage.getRaster();
			int[] pixelArray = new int[3];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					raster.getPixel(x, y, pixelArray);
					int pixel = palette.getPixel(new RGB(pixelArray[0],
							pixelArray[1], pixelArray[2]));
					data.setPixel(x, y, pixel);
				}
			}
			return data;
		} 
		else if (bufferedImage.getColorModel() instanceof IndexColorModel) {
			IndexColorModel colorModel = (IndexColorModel) bufferedImage
					.getColorModel();
			int size = colorModel.getMapSize();
			byte[] reds = new byte[size];
			byte[] greens = new byte[size];
			byte[] blues = new byte[size];
			colorModel.getReds(reds);
			colorModel.getGreens(greens);
			colorModel.getBlues(blues);
			RGB[] rgbs = new RGB[size];
			for (int i = 0; i < rgbs.length; i++) {
				rgbs[i] = new RGB(reds[i] & 0xFF, greens[i] & 0xFF,
						blues[i] & 0xFF);
			}
			PaletteData palette = new PaletteData(rgbs);
			ImageData data = new ImageData(bufferedImage.getWidth(),
					bufferedImage.getHeight(), colorModel.getPixelSize(),
					palette);
			data.transparentPixel = colorModel.getTransparentPixel();
			WritableRaster raster = bufferedImage.getRaster();
			int[] pixelArray = new int[1];
			for (int y = 0; y < data.height; y++) {
				for (int x = 0; x < data.width; x++) {
					raster.getPixel(x, y, pixelArray);
					data.setPixel(x, y, pixelArray[0]);
				}
			}
			return data;
		}
		return null;
	}

	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: BarcodeGenerator.java,v 1.1 2008-06-26 10:42:17 mario Exp $";
	}
}