/*
 * VatTypeDAO.java
 *
 * $Id: VatTypeDAO.java,v 1.2 2009-03-02 15:31:23 mario Exp $
 */
package com.imagos.axilice.server.common.dao;

import java.util.ArrayList;
import java.util.List;

import com.imagos.axilice.core.AxiliceException;
import com.imagos.axilice.core.InterfaceErrors;
import com.imagos.axilice.core.util.OperationResult;
import com.imagos.axilice.core.util.ResultMessage;
import com.imagos.axilice.core.common.VatTypeOT;
import com.imagos.axilice.server.ServerContext;
import com.imagos.axilice.server.util.db.CriteriaBean;
import com.imagos.axilice.server.util.db.HibernateUtil;
import com.imagos.axilice.server.util.db.SQLGenerator;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * This class implements the VAT type data access object. It's 
 * a part of the data access layer and gives access to the database data. 
 * It knows the database structure and implementation. 
 * <br/>
 * This class is implemented to access a PostgreSQL database implementation.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Provide statements for inserting new VAT types into the database. </ul>
 *  <ul> Provide statements for updating VAT types into the database. </ul>
 *  <ul> Provide statements for deleting VAT types from the database. </ul>
 *  <ul> Provide the database queries needed for VAT types operations.  </ul>
 * </li>
 */

public class VatTypeDAO {
	
	/**
	 * Default constructor of the VatTypeDAO class.
	 */
	public VatTypeDAO() {
	}
	
	/**
	 * Query the database to get all the available VAT types. 
	 * @throws Exception, in case any error take place during
	 * the query operation.
	 * @return ArrayList, list of all the VAT types.
	 */
	public List getAllVatTypes() throws Exception {
		List result = new ArrayList();
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();
		
        try {
        	// Load the objects
        	result = session.createQuery(
            "from VatTypeOT b")
            .list();

            System.out.println( "Consulta realizada! " +
            		result == null || result.size() <= 0?(" No se ha encontrado ningún VAT type!"):
            		("Se han encontrado: " + result.size() + " VAT types"));
        }
        catch (HibernateException ex) {
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().
        		newResultMessage(InterfaceErrors.ERR_VATTYPE_NOT_FOUND, ex);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query (by primary key) the database to get a VAT type data. 
	 * @param idVatType long, Internal id of the VAT type.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return VatTypeOT, VAT type if it exists, null in any other case.
	 */
	public VatTypeOT getVatType(long idVatType) throws AxiliceException {
		VatTypeOT result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	// Load the object by the primary key (idResource)
        	result = (VatTypeOT)session.get(VatTypeOT.class, new Long(idVatType));

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún VAT type correspondiente!"):
            		("Se ha encontrado el VAT type: " + result.getDescription()));
        }
        catch (HibernateException ex) {
        	ArrayList lParams = new ArrayList();
        	lParams.add("" + idVatType);
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_NOT_FOUND, ex, lParams);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query the database to get a VAT type data. 
	 * @param description String, VAT type description.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return VatTypeOT, VAT type if it exists, null in any other case.
	 */
	public VatTypeOT getVatType(String description) throws AxiliceException {
		VatTypeOT result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	// Build the query statement
        	String sql = "FROM VatTypeOT b WHERE b.description=?";
        	Query q = session.createQuery(sql);
        	q.setString(0, description);

        	// Run the query statement
        	result = (VatTypeOT)q.uniqueResult();

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún VAT type correspondiente!"):
            		("Se ha encontrado el VAT type: " + result.getDescription()));
        }
        catch (HibernateException ex) {
        	ArrayList lParams = new ArrayList();
        	lParams.add("" + description);
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_NOT_FOUND, ex, lParams);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query the database to get a list of VAT types that math
	 * the query criteria specified. 
	 * @param lCriteria ArrayList<CriteriaBean>, list of query criteria.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return ArrayList<VatTypeOT>, list of VAT types matching the 
	 * searching criteria.
	 */
	public ArrayList<VatTypeOT> getVatTypes(ArrayList<CriteriaBean> lCriteria) throws AxiliceException {
		ArrayList<VatTypeOT> result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	// Build the query statement
        	SQLGenerator sqlGen = new SQLGenerator();

        	Criteria crit = sqlGen.getSQLCriteria(session, VatTypeOT.class, lCriteria);
        	
        	// Run the query statement
        	result = (ArrayList)crit.list();

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún VAT type correspondiente!"):
            		("Se han encontrado VAT types: " + result.size()));
        }
        catch (HibernateException ex) {
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_NOT_FOUND, ex, null);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	public OperationResult delVatType(Long idVatType) throws AxiliceException {
		OperationResult result = new OperationResult();

		if (idVatType == null) {
			result.addResultMessage(OperationResult.RESULT_ERROR, 
					ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_DELETE_NULL));
		}
		else {
	
			// Build the list of parameters for the localized operation result message.
			ArrayList lParams = new ArrayList();
			lParams.add(idVatType);
		
			// Open a new Hibernate session to query the Database
			Session session = HibernateUtil.getSessionFactory().openSession();
	
	    	// Open a new transaction
	        Transaction tx = session.beginTransaction();
	
	        // Delete the VAT type from the database
	        try {
	        	VatTypeOT vatType = (VatTypeOT)session.load(VatTypeOT.class, idVatType);
	        	System.out.println("VAT Type (" + vatType.getIdVat() + ") found: " + vatType.getDescription());
	        	session.delete(vatType);
	        	tx.commit();
	        	
	        	lParams.add(idVatType);
	        	
	        	System.out.println("VatType (" + vatType.getIdVat() + ") deleted: " + vatType.getDescription());
	        	result.addResultMessage(OperationResult.RESULT_OK, 
	        			ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.MSG_VATTYPE_DELETE_OK, lParams));
	        }
	        catch (HibernateException ex) {
	        	ex.printStackTrace();
	        	tx.rollback();
	        	
	        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_DELETE, ex, lParams);
	        	throw new AxiliceException(msg);
	        }
	        
	        // Close the Hibernate session
	        session.close();
	        
	        System.out.println("Session closed!");
		} // else product!=null
        
        return result;
	}
	
	public OperationResult delVatTypes(ArrayList<Long> lSelVatTypes) throws AxiliceException {
		OperationResult result = new OperationResult();
		
        try {
        	// Build the query statement
        	for (int i=0; lSelVatTypes != null && i<lSelVatTypes.size(); i++) {
        		// Delete the VAT type
        		result.add(this.delVatType(lSelVatTypes.get(i)));
        	} // for each VAT type
        }
        catch (HibernateException ex) {
        	ex.printStackTrace();
        	
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_DELETE, ex, null);
        	throw new AxiliceException(msg);
        }
        
        return result;
	}
	
	public OperationResult saveVatType(VatTypeOT vatType) throws AxiliceException {
		OperationResult result = new OperationResult();

		if (vatType == null) {
			result.addResultMessage(OperationResult.RESULT_ERROR, 
					ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_SAVE_NULL));
		}
		else {
	
			// Build the list of parameters for the localized operation result message.
			ArrayList lParams = new ArrayList();
			lParams.add(vatType.getDescription());
		
			// Open a new Hibernate session to query the Database
			Session session = HibernateUtil.getSessionFactory().openSession();
	
	    	// Open a new transaction
	        Transaction tx = session.beginTransaction();
	
	        // Delete the VAT type from the database
	        try {
	        	session.saveOrUpdate(vatType);
	        	tx.commit();
	        	
	        	lParams.add(vatType.getIdVat());
	        	
	        	System.out.println("VatType (" + vatType.getIdVat() + ") saved: " + vatType.getDescription());
	        	result.addResultMessage(OperationResult.RESULT_OK, 
	        			ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.MSG_VATTYPE_SAVE_OK, lParams));
	        }
	        catch (HibernateException ex) {
	        	ex.printStackTrace();
	        	tx.rollback();
	        	
	        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceErrors.ERR_VATTYPE_SAVE, ex, lParams);
	        	throw new AxiliceException(msg);
	        }
	        
	        // Close the Hibernate session
	        session.close();
	        
	        System.out.println("Session closed!");
		} // else product!=null
        
        return result;
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: VatTypeDAO.java,v 1.2 2009-03-02 15:31:23 mario Exp $";
	}
}
