/*
 * MeasureUnitDAO.java
 *
 * $Id: MeasureUnitDAO.java,v 1.5 2009-07-14 09:28:43 mario Exp $
 */
package com.imagos.axilice.server.warehouse.dao;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import com.imagos.axilice.core.AxiliceException;
import com.imagos.axilice.core.util.OperationResult;
import com.imagos.axilice.core.util.ResultMessage;
import com.imagos.axilice.core.warehouse.InterfaceWarehouse;
import com.imagos.axilice.core.warehouse.InterfaceWarehouseErrors;
import com.imagos.axilice.core.warehouse.MeasureUnitOT;
import com.imagos.axilice.server.ServerContext;
import com.imagos.axilice.server.util.db.CriteriaBean;
import com.imagos.axilice.server.util.db.HibernateUtil;
import com.imagos.axilice.server.util.db.SQLGenerator;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * This class implements the measure unit data access object. It's 
 * a part of the data access layer and gives access to the database data. 
 * It knows the database structure and implementation. 
 * <br/>
 * This class is implemented to access a PostgreSQL database implementation.
 * <br/>
 * Responsibilities:
 * <li>
 * 	<ul> Provide statements for inserting new measure units into the database. </ul>
 *  <ul> Provide statements for updating measure units into the database. </ul>
 *  <ul> Provide statements for deleting measure units from the database. </ul>
 *  <ul> Provide the database queries needed for measure units operations.  </ul>
 * </li>
 */

public class MeasureUnitDAO {
	
	/**
	 * Default constructor of the ProductDAO class.
	 */
	public MeasureUnitDAO() {
	}
	
	/**
	 * Query the database to get all the available measure units. 
	 * @throws Exception, in case any error take place during
	 * the query operation.
	 * @return ArrayList, list of all the measure units.
	 */
	public List getAllMeasureUnits() throws Exception {
		List result = new ArrayList();
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();
		
        try {
        	// Load the objects
        	result = session.createQuery(
            "from MeasureUnitOT mu")
            .list();

            System.out.println( "Consulta realizada! " +
            		result == null || result.size() <= 0?(" No se ha encontrado ningún unidad de medida!"):
            		("Se han encontrado: " + result.size() + " unidad de medidas"));
        }
        catch (HibernateException ex) {
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_MEASUREUNIT_NOT_FOUND, ex);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query (by primary key) the database to get a measure unit data. 
	 * @param idMeasureUnit long, Internal id of the measure unit.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return MeasureUnitOT, measure unit if it exists, null in any other case.
	 */
	public MeasureUnitOT getMeasureUnit(long idMeasureUnit) throws AxiliceException {
		MeasureUnitOT result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	// Load the object by the primary key (idResource)
        	result = (MeasureUnitOT)session.get(MeasureUnitOT.class, new Long(idMeasureUnit));

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún producto correspondiente!"):
            		("Se ha encontrado el producto: " + result.getDescription()));
        }
        catch (HibernateException ex) {
        	ArrayList lParams = new ArrayList();
        	lParams.add("" + idMeasureUnit);
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_MEASUREUNIT_NOT_FOUND, ex, lParams);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query the database to get a measure unit data. 
	 * @param description String, measure unit description.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return MeasureUnitOT, measure unit if it exists, null in any other case.
	 */
	public MeasureUnitOT getMeasureUnit(String description) throws AxiliceException {
		MeasureUnitOT result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	// Build the query statement
        	String sql = "FROM MeasureUnitOT mu WHERE mu.description=?";
        	Query q = session.createQuery(sql);
        	q.setString(0, description);

        	// Run the query statement
        	result = (MeasureUnitOT)q.uniqueResult();

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún unidad de medida correspondiente!"):
            		("Se ha encontrado el unidad de medida: " + result.getDescription()));
        }
        catch (HibernateException ex) {
        	ArrayList lParams = new ArrayList();
        	lParams.add("" + description);
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_MEASUREUNIT_NOT_FOUND, ex, lParams);
        	throw new AxiliceException(msg);
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	/**
	 * Query the database to get a list of measure units that math
	 * the query criteria specified. 
	 * @param lCriteria ArrayList<CriteriaBean>, list of query criteria.
	 * @throws AxiliceException, in case any error take place during
	 * the query operation.
	 * @return ArrayList<MeasureUnitOT>, list of measure units matching the 
	 * searching criteria.
	 */
	public ArrayList<MeasureUnitOT> getMeasureUnits(ArrayList<CriteriaBean> lCriteria) throws AxiliceException {
		ArrayList<MeasureUnitOT> result = null;
		// Open a new Hibernate session to query the Database
		Session session = HibernateUtil.getSessionFactory().openSession();

        try {
        	System.out.println("Consultando measure-units: " + lCriteria);
        	ServerContext.getInstance().getLogger().debug("Consultando measure-units: " + lCriteria);
        	// Build the query statement
        	SQLGenerator sqlGen = new SQLGenerator();
        	Criteria crit = sqlGen.getSQLCriteria(session, MeasureUnitOT.class, lCriteria);
        	
        	// Run the query statement
        	result = (ArrayList)crit.list();

            System.out.println( "Consulta realizada! " +
            		result == null?(" No se ha encontrado ningún unidad de medida correspondiente!"):
            		("Se han encontrado unidades de medida: " + result.size()));
        }
        catch (HibernateException ex) {
/*        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_MEASUREUNIT_NOT_FOUND, ex, null);
        	throw new AxiliceException(msg);
*/
        	ex.printStackTrace();
        }
        
        // Close the Hibernate session
        session.close();	
        
        // Return the result of the query
        return result;
	}
	
	public OperationResult delMeasureUnit(Long idMeasureUnit) throws AxiliceException {
		OperationResult result = new OperationResult();
		
		if (idMeasureUnit == null) {
			result.addResultMessage(OperationResult.RESULT_ERROR, 
					ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_DELETE_MEASUREUNIT_NULL));
		}
		else {
			// Open a new Hibernate session to query the Database
			Session session = HibernateUtil.getSessionFactory().openSession();
	
			// Build the list of parameters for the localized operation result message.
			ArrayList lParams = new ArrayList();
	    	lParams.add(idMeasureUnit);
    	
	    	// Open a new transaction
	        Transaction tx = session.beginTransaction();
	
	        // Delete the product from the database
	        try {
	        	MeasureUnitOT group = (MeasureUnitOT)session.load(MeasureUnitOT.class, idMeasureUnit);
	        	System.out.println("Product Group (" + group.getIdMeasureUnit() + ") found: " + group.getDescription());
	        	
	        	session.delete(group);
	        	tx.commit();
	        	
	        	System.out.println("Product Group (" + group.getIdMeasureUnit() + ") deleted: " + group.getDescription());
	        	result.addResultMessage(OperationResult.RESULT_OK, 
	        			ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.MSG_WAREHOUSE_DELETE_MEASUREUNIT_OK, lParams));
	        }
	        catch (HibernateException ex) {
	        	ex.printStackTrace();
	        	tx.rollback();
	        	
	        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_DELETE_MEASUREUNIT, ex, lParams);
	        	throw new AxiliceException(msg);
	        }
	        
	        // Close the Hibernate session
	        session.close();
	        
	        System.out.println("Session closed!");
		}
        
        return result;
	}
	
	public OperationResult delMeasureUnits(ArrayList<Long> lSelGroups) throws AxiliceException {
		OperationResult result = new OperationResult();
		
        try {
        	// Build the query statement
        	for (int i=0; lSelGroups != null && i<lSelGroups.size(); i++) {
        		// Delete the group
        		result.add(this.delMeasureUnit(lSelGroups.get(i)));
        	} // for each group
        }
        catch (HibernateException ex) {
        	ex.printStackTrace();
        	
        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_DELETE_MEASUREUNIT, ex, null);
        	throw new AxiliceException(msg);
        }
        
        return result;
	}
	
	public OperationResult saveMeasureUnit(MeasureUnitOT group) throws AxiliceException {
		OperationResult result = new OperationResult();

		if (group == null) {
			result.addResultMessage(OperationResult.RESULT_ERROR, 
					ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_SAVE_MEASUREUNIT_NULL));
		}
		else {
			// Build the list of parameters for the localized operation result message.
			ArrayList lParams = new ArrayList();
			lParams.add(group.getDescription());
		
			// Open a new Hibernate session to query the Database
			Session session = HibernateUtil.getSessionFactory().openSession();
	
	    	// Open a new transaction
	        Transaction tx = session.beginTransaction();
	
	        // Delete the product from the database
	        try {
	        	session.saveOrUpdate(group);
	        	tx.commit();
	        	
	        	System.out.println("Product (" + group.getIdMeasureUnit() + ") saved: " + group.getDescription());
	        	result.addResultMessage(OperationResult.RESULT_OK, 
	        			ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.MSG_WAREHOUSE_SAVE_MEASUREUNIT_OK, lParams));
	        }
	        catch (HibernateException ex) {
	        	ex.printStackTrace();
	        	tx.rollback();
	        	
	        	ResultMessage msg = ServerContext.getInstance().getMsgCatalog().newResultMessage(InterfaceWarehouseErrors.ERR_WAREHOUSE_SAVE_MEASUREUNIT, ex, lParams);
	        	throw new AxiliceException(msg);
	        }
	        
	        // Close the Hibernate session
	        session.close();
	        
	        System.out.println("Session closed!");
		} // else product!=null
        
        return result;
	}
	
	/**
	 * Returns tag Id assigned to CVS source file.
	 */
	public static String getRevision() {
		return "$Id: MeasureUnitDAO.java,v 1.5 2009-07-14 09:28:43 mario Exp $";
	}
}

